// Demo 15: ArrayDemo.java

//             A Simple Array Demo
//         A Screen Based Application
//
//         by Charles W. Neville, 1998
//      Copyright Charles W. Neville, 1998

// An ARRAY is, depending on your taste,
//   (1) A vector (as in vectors in mathematics),
//   (2) A finite sequence (as in x , x , x , ..., x  in mathematics),
//                                 1   2   3        n
//   (3) A row of boxes.
// Let us take the third point of view, that an array is a row of boxes.  We 
// might declare and size an array containing three integers this way:
//
//    int[] A;
//    A = new int[3];
//
// After a program involving A ran for some time, the array A might look 
// something like this:
//              ----- ----- -----
//    array A  | 243 | 505 | 252 |
//              ----- ----- -----
//    index i     0     1     2
//
// (Notice that the boxes are numbered starting from 0, not 1, so the highest 
// numbered box of an array of size 3 is box 2, not box 3.)  We would refer to 
// box 1 by A[1], so A[1] would have the value 505.  Similarly A[0] would have 
// the value 243 and A[2] would have the value 252.
//
// The great advantage of arrays is that we can access their boxes by NUMBER, 
// and that number can even be stored in a variable or computed by an 
// expression.  Thus the following block of code involving a for loop prints 
// out the contents of array A:
//
//    for (int i = 0; i < 3; i++) {
//
//        System.out.println(A[i]);
//    }
//
// This may not seem very important when dealing with an array only 3 boxes 
// long, but it becomes EXTREMELY USEFUL when dealing with an array 1000 boxes 
// long.  (If you don't believe me, try printing out the values of 1000 
// separate variables with 1000 separate System.out.println statements.)
//
// In the following Array Demo, we shall take advantage of this NUMERIC method 
// of accessing array boxes to do things like print a list of names out 
// backwards.  We shall also illustrate how declare and size an array, how to 
// fill the boxes of an array with data, how to determine the length of an 
// array (very useful for writing general code) and how to print out an array 
// of names in forward order. 

class ArrayDemo {

   public static void main(String[] args) {

      // declare an array of Strings.  (Each box contains a string.)
      String[] mySentence;

      // declare another array of Strings
      String[] hamlet;

      // Now you know what the mysterious String[] args part means in
      // public static void main( String[] args ).  args is an ARRAY of 
      // strings, namely the array of command line arguments entered when the 
      // program was run.  Later, we will see how to use these to make a Java 
      // application behave differently, depending on what command line 
      // arguments the user entered.

      // create the array and size it so it is 6 boxes long.
      mySentence = new String[6];

      // fill the array with words by hand
      mySentence[0] = "To";
      mySentence[1] = "be";
      mySentence[2] = "or";
      mySentence[3] = "not";
      mySentence[4] = "to";
      mySentence[5] = "be";

      // Note: You should be able to DRAW a picture of the array mySentence at 
      //       this point, one similar to the picture of the array A in the 
      //       remarks preceeding the program.

      // get the length of the mySentence array and print it out
      System.out.println("length of mySentence is " + mySentence.length);

      // skip a line
      System.out.println();

      // print out the mySentence array
      System.out.println("mySentence is");
      for (int i = 0; i < mySentence.length; i++) {

          System.out.println(mySentence[i]);
      }

      // What does the demo print out at this point?  Run it and see.

      System.out.println();  // skip another line

      // print out the mySentence array backwards
      System.out.println("mySentence backwards is");
      for (int i = mySentence.length -1; i >= 0; i--) {

          System.out.println(mySentence[i]);
      }

      System.out.println();  // skip another line

      // This is subtle.  We can assign one array variable to another like 
      // this:
      hamlet = mySentence;

      // But if we then change mySentence, hamlet will also be changed!

      mySentence[1] = "program";
      mySentence[5] = "program";

      System.out.println("mySentence now is");
      for (int i = 0; i < mySentence.length; i++) {

          System.out.println(mySentence[i]);
       }

      System.out.println();  // skip a line

      System.out.println("hamlet now is");
      for (int i = 0; i < hamlet.length; i++) {

          System.out.println(hamlet[i]);
      }

      // This is the exact opposite of what happens when we assign one scalar 
      // type variable to another.  (The scalar types are boolean, int, float, 
      // double, char, etc.  They represent one number, character or bit. 
      // Arrays are NOT scalar types.)  For example,  if we do the following to 
      // two variables of type int, say,
      //
      //     i = 5; j = i;  i = 4;
      //     System.out.println("i = " + i + " and " + "j = " + j);
      //
      // we will get i = 4 and j = 5.  Arrays are different, aren't they!

      System.out.println();  // skip another line

      System.out.println("Arrays are different, aren't they!");
   }
}
